/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 */

var _sgTodo = {};

var _theList = false;
var _doneList = null;
var _nextid = 1;
var _idToGuid = {};
var _guidToId = {};
var _todoQueue = new ajaxQueue( { saveQueue: true} );

function _addBelow(guid, prev, title, status, totalwork)
{
	var n = _nextid++;
	var ourid = "vvtodo" + n;

	var li = $(document.createElement('li')).
		addClass(status).
		attr('id', ourid);
	_idToGuid[ourid] = guid;
	_guidToId[guid] = ourid;

	var sp = $(document.createElement('span')).
		text(' ').
		addClass('handle');
	li.append(sp);

	li.draggable( { containment: 'parent' } );

	var cb = $(document.createElement('input')).
		attr('type', 'checkbox');

	if (status && (status != 'open'))
		cb.attr('checked', 'checked');

	cb.click(
		function (e) {
			var newStatus = '';
			if (cb.is(':checked'))
			{
				li.removeClass('verified');
				li.removeClass('open');
				li.addClass('fixed');

				newStatus = 'fixed';

				_dropToBottom(li);
			}
			else
			{
				li.removeClass('verified');
				li.removeClass('fixed');
				li.addClass('open');
				newStatus = 'open';

				$('#thelist').prepend(li);

				_backToList(li);
			}

			guid = guid || _idToGuid[ourid];

			_updateItemField(guid, 'status', newStatus, inp);
		}
	);

	li.append(cb);

	var inp = $(document.createElement('textarea')).
		val(title).
		attr('rows', 1).
		dblclick(
			function(e) {
				e.preventDefault();

				var g = _idToGuid[ourid];
				if (g) {
					var u = sgCurrentRepoUrl + '/workitem.html?recid=' + g;
					window.document.location.href = u;
				}
			}
		);

	inp.data('originalvalue', title);

	var checker = function()
	{
		if (inp.val() != inp.data('originalvalue'))
		{
			_todoRetryShow();
			inp.data('dirty', true).addClass('tododirty');
		}
	};

	inp.keyup(checker);
	inp.change(checker);

	inp.keydown(
			function(e) {
				var moved = false;

				switch (e.which || e.keyCode) {
				case 13:
					e.preventDefault();
					e.stopPropagation();

					if (jQuery.trim(inp.val()) != "") {
						inp.data('originalvalue', inp.val());

						_updateItem(li);

						_addAtEnd();
					}
					break;

				case 38: // up
					e.preventDefault();
					e.stopPropagation();

					if (li.prev())
					{
						if (e.ctrlKey)	// move to top
							li.siblings().first().before(li);
						else 			// move up
							li.prev().before(li);

						moved = true;
					}

					break;

				case 40: // down
					e.preventDefault();
					e.stopPropagation();

					if (li.next())
					{
						if (e.ctrlKey)
							li.siblings().last().after(li);
						else
							li.next().after(li);

						while (li.prev() && li.prev().find('textarea').val() == '')
							li.prev().before(li);

						moved = true;
					}
					break;
				}

				if (moved)
				{
					_juggleOrder(_orderedGuidList(), _sgTodo);
					inp.focus();
				}
			}
	);

	inp.blur(
			function(e) {
				_updateItemField(guid, 'title', inp.val(), inp);
			}
	);

	li.append(inp);

	if (prev)
		prev.after(li);
	else
		_theList.append(li);

	// do this here so we get resized right away
	autoResizeText(inp, 2000);

	return(li);
}

function _backToList(li)
{
	var next = li.next('li');
	var ourid = li.attr('id');
	if ((! ourid) || (! _idToGuid[ourid]))
		return;
	var guid = _idToGuid[ourid];
	var rec = _sgTodo[guid];
	var lo = -1;

	if (rec)
		lo = rec.listorder;
	var lastLess = null;

	while (next.length != 0)
	{
		var id = next.attr('id');
		guid = _idToGuid[id];
		rec = _sgTodo[guid];

		if (! rec)
			break;

		if (lo > rec.listorder)
			lastLess = next;
		else
			break;

		next = next.next('li');
	}

	if (lastLess)
		lastLess.after(li);

}

function _updateItemField(guid, vari, value, inp, stall)
{
	var rec = _sgTodo[guid] || null;

	if (rec && (rec[vari] !== value))
	{
		if ((rec.assignee == sgUserName) && (rec.verifier == sgUserName) && (vari == 'status') && (value == 'fixed'))
			value = 'verified';

		rec[vari] = value;

		if (inp)
			inp.data('dirty', true).addClass('tododirty');
		_todoRetryShow();

		_updateWorkItem(guid, inp, stall);
	}
	else if ((! guid) && inp.val())
	{
		_todoRetryHide();
		_updateItem(inp.closest('li'));
		_addAtEnd();
	}
}


function _updateWorkItem(guid, inp, stall) {
	var item = _sgTodo[guid];

	var uurl = sgCurrentRepoUrl + '/workitem/' + guid + "?_returnrecord=1";

	var updateItem = {
		title: item.title
	};

	if (item.listorder)
		updateItem.listorder = parseInt(item.listorder);
	if (item.priority !== undefined)
		updateItem.priority = item.priority;
	if (item.status)
		updateItem.status = item.status;
	if (item._csid)
		updateItem._csid = item._csid;

	var j = JSON.stringify(updateItem);

	_todoRetryShow();

	if (inp)
		inp.addClass('todoinprogress').addClass('tododirty');

	var opts = {
			url: uurl,
			dataType: 'json',
			contentType: 'application/json',
			processData: false,
			type: 'PUT',
			data: j,
			errorMessage: "An error occurred while attempting to save your changes. Please click 'Save' to retry.",

			success: function(data) {
				if (inp)
				{
					inp.removeClass('todoinprogress').
						removeClass('tododirty').
						data('dirty', false);
				}
				if (data)
					_sgTodo[guid] = data;
			},

			error: function() {
				inp.removeClass('todoinprogress');
				_todoQueue.addQueueRequest( opts );
				_todoRetryNeeded();
			}
		};

	_todoQueue.addQueueRequest( opts );

	if (! stall)
	{
		_todoRetryHide();
		_todoQueue.execute();
	}
}

function _todoRetryHide()
{
	$('#todoretry').addClass('disabled').attr('disabled', 'disabled');
}

function _todoRetryShow()
{
	$('#todoretry').removeClass('disabled').attr('disabled', false);
}

function _todoRetryNeeded()
{
	var dt = _dirtyTodos();

	if (dt)
		$.each(dt, function(i,el) {
				   $(el).removeClass('todoinprogress');
			   }
			  );

	_todoRetryShow();
}

function _addAtEnd()
{
	var lastOne = false;
	var lastSeen = 0;
	var lastInList = false;

	$('#thelist li').each(
			function() {
				var id = $(this).attr('id');
				var g = _idToGuid[id];
				lastInList = $(this);

				if (g && _sgTodo[g])
				{
					var o = _num(_sgTodo[g].listorder);
					if (o > lastSeen)
						lastSeen = o;
				}

				var tv = $(this).children('textarea').val();

				if ($.trim(tv) == "")
				{
					lastOne = this;
				}
			}
	);

	if (! lastOne)
	{
		_addBelow('', lastInList, '', 'open').children('textarea').focus();
	}
}


function _initSprintList(selector)
{
	var cname = vCore.getDefaultCookieName();
	var ival = vCore.getCookieValueForKey(cname, "todoSprint", null) || sgCurrentSprint || false;

	var _addToList = function(node, level)
	{
		var prefix = "";

		for ( var i = 0; i < level; ++i )
			prefix += "-";
		if (level > 0)
			prefix += " ";

		var o = $(document.createElement('option')).
			val(node.key).
			text(prefix + node.title).
			appendTo(selector);

		if (ival)
		{
			if (node.key == ival)
				o.attr('selected', 'selected');
		}
		else
			if (! node.key)
				o.attr('selected', 'selected');

		var kids = node.children || [];

		for ( i = 0; i < kids.length; ++i )
			_addToList(kids[i], level + 1);
	};

	vCore.fetchSprints(
		{
			success: function(sprints, status, xhr)
			{
				var tree = new vCore.milestoneTree(sprints);

				_addToList(tree.root, 0);

				var ua = tree.unattachedNodes || [];

				for ( var i = 0; i < ua.length; ++i )
					_addToList(ua[i], 0);

				selector.change(
					function (e) {
						vCore.setCookieValueForKey(cname, "todoSprint", selector.val(), 30);
					}
				);

			},
			releaseOnly: true,
			force: true
		}
	);
}

function _initPriorityList(selector)
{
	var priorities = [ "High", "Medium", "Low" ];
	var cname = vCore.getDefaultCookieName();
	var ival = vCore.getCookieValueForKey(cname, "todoPriority", null);

	for ( var i in priorities )
	{
		var s = priorities[i];

		var o = $(document.createElement('option')).
			val(s).
			text(s).
			appendTo(selector);

		if (ival && (s == ival))
			o.attr('selected', 'selected');
	}

	selector.change(
		function (e) {
			vCore.setCookieValueForKey(cname, "todoPriority", selector.val(), 30);
		}
	);
}

function _filterList()
{
	var lis = _theList.find('li');
	var sprint = $('#sprintfilter').val();
	var priority = $('#priorityfilter').val();


	$(lis).each(function(i, li) {
		var show = true;
		li = $(li);

		var g = _idToGuid[li.attr('id')];
		var w = _sgTodo[g];

		if (w)
		{
			if (sprint)
				show = (w.milestone == sprint);
			else
				show = (! w.milestone);
			if (priority && show)
				show = (w.priority == priority);

			if (show)
				li.show();
			else
				li.hide();
		}
	});
}

function _reloadList()
{
	var s = $('#sprintfilter').val() || '';
	var p = $('#priorityfilter').val() || '';

	_loadList(s, p);
}

function _loadList(sprint, priority)
{
	var purl = sgCurrentRepoUrl + '/workitems.json?assignee=' + encodeURIComponent(sgUserName) + '&_fields=*,!&status=open';

	var cname = vCore.getDefaultCookieName();

	if (typeof(sprint) == 'undefined')
		sprint = vCore.getCookieValueForKey(cname, "todoSprint", null) || sgCurrentSprint || '';
	if (typeof(priority) == 'undefined')
		priority = vCore.getCookieValueForKey(cname, "todoPriority", null);

	purl += '&milestone=' + encodeURIComponent(sprint);
	if (priority)
		purl += '&priority=' + encodeURIComponent(priority);

	var prev = false;
	var guidsInOrder = [];
	_theList.empty();
	_idToGuid = {};
	_guidToId = {};
	_nextid = 1;

	vCore.ajax(
		{
			url: purl,
			dataType: 'json',

			success: function(wits) {

				for (var i in wits)
				{
					var wit = wits[i];

					guidsInOrder.push(wit.recid);
					_sgTodo[wit.recid] = wit;
				}

				_sortByOrder(guidsInOrder);

				for ( i in guidsInOrder )
				{
					var g = guidsInOrder[i];
					var item = _sgTodo[g];

					prev = _addBelow(g, prev, item.title, item.status, item.estimate);
				}

				prev = _addBelow('', prev, '', 'open');

				_filterList();
				_juggleOrder(guidsInOrder, _sgTodo);
				$(prev).children('textarea').focus();
			}
		});
}


function _initList(container) {
	_initSprintList($('#sprintfilter'));
	_initPriorityList($('#priorityfilter'));

	$('#sprintfilter').change( _reloadList );
	$('#priorityfilter').change( _reloadList );

	_theList = $(document.createElement('ul')).
		addClass('todolist').
		attr('id','thelist').
		empty().
		appendTo(container);

	_doneList = $(document.createElement('ul')).
		addClass('todolist').
		appendTo(container);

	_loadList();

	_theList.droppable(
		{
			drop: function(e, ui) {
			var dragged = ui.draggable;

			var lastBefore = false;
			var firstAfter = false;

			var dtop = $(dragged).position().top;

			_theList.children().each(
					function () {
						var el = $(this);

						if (el.attr('id') == $(dragged).attr('id'))
							return;
						if (! el.is(':visible'))
							return;

						var ctop = el.position().top;

						if (ctop < dtop)
						{
							lastBefore = el;
						}
						else if ((ctop >= dtop) && ! firstAfter)
						{
							firstAfter = el;
						}
					}
			);

			if (lastBefore)
				$(lastBefore).after(dragged);
			else if (firstAfter)
				$(firstAfter).before(dragged);

			$(dragged).css('top' ,'auto');
			$(dragged).css('left', 'auto');

			_juggleOrder(_orderedGuidList(), _sgTodo);
		}
	});
}


function _priorityVal(item)
{
	if (item)
	{
		if (item.priority == 'High')
			return(0);
		else if (item.priority == 'Medium')
			return(1);
		else if (item.priority == 'Low')
			return(2);
	}

	return(3);
}


function _sortByOrder(items)
{
	items.sort(
			function(ga, gb) {
				var a = _sgTodo[ga];
				var b = _sgTodo[gb];

				if (a.listorder && b.listorder)
				{
					a.listorder = _num(a.listorder);
					b.listorder = _num(b.listorder);
					return(a.listorder - b.listorder);
				}
				else if (a.listorder)
				{
					return(-1);
				}
				else if (b.listorder)
				{
					return(1);
				}
				else
				{
					return( _priorityVal(a) - _priorityVal(b) );
				}
			}
	);
}

function _orderedGuidList()
{
	var list = [];

	$('#todolist li').each(
			function() {

				var id = $(this).attr('id');
				var g = _idToGuid[id];
				list.push(g);
			}
	);

	return(list);
}

function _createWorkItem(listid, textval, inp)
{
	if (inp.data('added'))
		return;

	inp.data('added', true);

	var purl = sgCurrentRepoUrl + '/workitems';

	var wi = {
		rectype: 'item',
		title: textval,
		status: "open",
		assignee: sgUserName,
		verifier: sgUserName,
		stamps: [ {'name': 'todo' } ]
	};

	var defSprint = $('#sprintfilter').val();
	var defPriority = $('#priorityfilter').val();

	if (defSprint)
		wi.milestone = defSprint;
	if (defPriority)
		wi.priority = defPriority;

	var nextli = _juggleOrder(_orderedGuidList(), _sgTodo);

	wi.listorder = nextli + 5;

	var j = JSON.stringify(wi);

	if (inp)
		inp.addClass('todoinprogress').addClass('tododirty');

	var req =
	{
		url: purl,
		dataType: 'text',
		contentType: 'application/json',
		processData: false,
		type: 'POST',
		data: j,
		errorMessage: "An error occurred while attempting to save your changes. Please click 'Save' to retry.",

		success: function(data) {
			if (inp)
			{
				inp.removeClass('todoinprogress').
					removeClass('tododirty').
					data('dirty', false);
			}

			_idToGuid[listid] = data;
			_guidToId[data] = listid;

			_sgTodo[data] = wi;

			_juggleOrder(_orderedGuidList(), _sgTodo);
		},

		error: function() {
			inp.removeClass('todoinprogress');
			_todoQueue.addQueueRequest( opts );
			_todoRetryNeeded();
		}
	};

	_todoQueue.addQueueRequest(req);

	_todoRetryHide();
	_todoQueue.execute();
}

function _updateItem(li)
{
	var oldId = $(li).attr('id');
	var inp = $(li).children('textarea');
	var val = inp.val();
	val = jQuery.trim(val);

	var g = _idToGuid[oldId];

	if (! g)
	{
		_createWorkItem(oldId, val, inp);
	}
	else
	{
		_updateItemField(g, 'title', val, inp);
	}
}

function _juggleOrder(orderedGuids, items)
{
	var lastSeen = 0;
	var mods = [];
	var inps = [];

	for (i in orderedGuids)
	{
		var g = orderedGuids[i];

		if (g && items[g])
		{
			var id = _guidToId[g];
			var vis = true;

			if (id)
				vis = $('#' + id).is(':visible');

			if (vis)
			{
				if (_num(items[g].listorder) <= lastSeen)
				{
					var inp = null;
					if (id)
					{
						inp = $('#' + id + ' textarea');
						inp.addClass('tododirty').addClass('todoinprogress');
						inps.push(inp);
					}

					items[g].listorder = lastSeen + 1;

					var r = {
						'recid': g,
						'listorder': lastSeen + 1
					};
					mods.push(r);
				}

				lastSeen = _num(items[g].listorder);
			}
		}
	}

	var opts = {
		url: sgCurrentRepoUrl + "/workitem-batch-update",
		dataType: 'json',
		contentType: 'application/json',
		processData: false,
		type: 'PUT',
		data: JSON.stringify(mods),
		errorMessage: "An error occurred while attempting to save your changes. Please click 'Save' to retry.",

		success: function(data) {
			$.each(
				inps,
				function(i,inpp){
					$(inpp).removeClass('todoinprogress').
						removeClass('tododirty').
						data('dirty', false);
				}
			);

			if (data)
			{
				for ( var i = 0; i < data.length; ++i )
				{
					var recid = data[i].recid;
					_sgTodo[recid] = data[i];
				}
			}
		},

		error: function() {
			$.each(
				inps,
				function(i,inpp){
					$(inpp).removeClass('todoinprogress');
				}
			);

			_todoQueue.addQueueRequest( opts );
			_todoRetryNeeded();
		}
	};

	if (mods.length > 0)
	{
		_todoRetryHide();
		_todoQueue.addQueueRequest(opts);
		_todoQueue.execute();
	}

	return(lastSeen + 1);
}


function _num(val)
{
	if (! val)
		return(0);

	if (typeof val == "string")
		return(parseInt(val));

	return(val);
}

function _dropToBottom(li)
{
	_doneList.append(li);
}

function _dirtyTodos()
{
	var any = null;

	$('#todolist textarea').each(
		function(i,e)
		{
			if ($(e).data('dirty'))
			{
				any = any || [];
				any.push($(e));
			}
		}
	);

	return(any);
}

$(document).ready(
	function ()
	{
		var tt = _toggler(sgStaticLinkRoot + "/img/opengreen.png",
						  sgStaticLinkRoot + "/img/closegreen.png",
						  [$('#todohelp')], false);
		$('#helpswitch').append(tt);

		vCore.oldbusy = vCore.vvIsBusy;
		vCore.vvIsBusy = function(e)
		{
			return( vCore.oldbusy(e) ||
					(_todoQueue.writesPending() > 0) ||
					_dirtyTodos()
				  );
		};

		var tdr = $('#todoretry');

		tdr.click(
			function(e)
			{
				e.stopPropagation();
				e.preventDefault();

				var dt = _dirtyTodos();

				if (dt)
					$.each(dt, function(i,el) {
							    $(el).addClass('todoinprogress');
						   }
						  );

				_todoRetryHide();
				_todoQueue.execute();

				return(false);
			}
		);

		_initList(document.getElementById('todolist'));
	}
);
