/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

#include "precompiled.h"
#include "vvRevDetailsDialog.h"

#include "vvVerbs.h"
#include "vvValidatorEmptyConstraint.h"
#include "vvValidatorRepoExistsConstraint.h"
#include "vvProgressExecutor.h"
#include "vvInOutCommand.h"
#include "vvDialogHeaderControl.h"
#include "vvAuditDetailsControl.h"
#include "vvRevisionLinkControl.h"
#include "vvHistoryObjects.h"

/*
**
** Public Functions
**
*/

bool vvRevDetailsDialog::Create(
	wxWindow*            pParent,
	const wxString&      sFolder,
	const wxString&      sRepoName,
	const wxString&      sRevID,
	vvVerbs&             cVerbs,
	vvContext&           cContext
	)
{
	// create the dialog window
	if (!vvDialog::Create(pParent, &cVerbs, &cContext, this->GetClassInfo()->GetClassName()))
	{
		return false;
	}
	// create our header control
	wxXmlResource::Get()->AttachUnknownControl("wHeader", new vvDialogHeaderControl(this, wxID_ANY, sFolder, cVerbs, cContext));
	msFolder = sFolder;
	msRepoName = sRepoName;
	msRevID = sRevID;
	m_nParentRevLinkID_1 = wxWindow::NewControlId();
	m_nParentRevLinkID_2 = wxWindow::NewControlId();
	vvRevisionLinkControl * pRevLink = new vvRevisionLinkControl(0, "", this, m_nParentRevLinkID_1);
	wxXmlResource::Get()->AttachUnknownControl("wRevLinkControl", pRevLink, this);
	this->Layout();
	this->Fit();
	return true;
}

void vvRevDetailsDialog::RevLinkClicked(wxCommandEvent& evt)
{
	evt;
	if (evt.GetString() != wxEmptyString)
	{
		this->LoadRevision(evt.GetString());
	}
	evt.Skip();
}

void vvRevDetailsDialog::LoadRevision(wxString sRevHID)
{
	vvContext&     pCtx = *this->GetContext();
	vvVerbs*   pVerbs   = this->GetVerbs();
	vvVerbs::HistoryEntry cEntry;
	vvVerbs::RevisionList children;
	pVerbs->Get_Revision_Details(pCtx, msRepoName, sRevHID, cEntry, children);
	vvRevisionLinkControl * pRevLink = XRCCTRL(*this, "wRevLinkControl",                 vvRevisionLinkControl);
	//vvRevisionLinkControl * pRevLink = (vvRevisionLinkControl *)wxWindow::FindWindowById(m_nParentRevLinkID_1, this);
	if (pRevLink != NULL)
	{
		if (cEntry.cParents.size() > 0)
		{
			pRevLink->SetRevision(cEntry.cParents[0].nRevno, cEntry.cParents[0].sChangesetHID);
			pRevLink->Show();
		}
		else
			pRevLink->Hide();
	}


	wxScrolledWindow * pSW = XRCCTRL(*this, "wAudits",                 wxScrolledWindow);
	pSW->DestroyChildren();
	wxBoxSizer * sizer = new wxBoxSizer(wxVERTICAL);
	for (unsigned int i = 0; i < cEntry.cAudits.size(); i ++)
	{
		sizer->Add(new vvAuditDetailsControl(*pVerbs, pCtx, cEntry.cAudits[i], cEntry.cComments.size() > i ? cEntry.cComments[i] : "", pSW, wxID_ANY), 1, wxEXPAND);
	}
	pSW->SetSizer(sizer);
	pSW->SetScrollRate(10, 10);
	this->Layout();
	//this->Fit();
	
}

bool vvRevDetailsDialog::TransferDataToWindow()
{ 
	
	this->LoadRevision(msRevID);
	return true;
}

bool vvRevDetailsDialog::TransferDataFromWindow()
{
	
	return true;
}

IMPLEMENT_DYNAMIC_CLASS(vvRevDetailsDialog, vvDialog);
BEGIN_EVENT_TABLE(vvRevDetailsDialog, vvDialog)
	//EVT_REVISION_LINK_CLICKED(wxID_ANY, vvRevDetailsDialog::RevLinkClicked)
	EVT_BUTTON(wxID_ANY, vvRevDetailsDialog::RevLinkClicked)
END_EVENT_TABLE()
