/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Resolve Tests
 *
 * This file contains tests that setup collision conflicts.
 */

// a test that creates colliding files via renaming
// and accepts the state of all items from "ancestor"
TEST("collision__rename__ancestor")
	TEST_SETUP
		SETUP_ADD_FILE("@/file1.txt", NULL, NULL, NULL, NULL)
		SETUP_ADD_FILE("@/file2.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file2.txt", "collision.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file1.txt", "collision.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file1.txt", "@/collision.txt", "@/file1.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file1.txt",           "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "collision.txt",       "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file1.txt",           "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision.txt~L0~g~", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file2.txt", "@/file2.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file2.txt",           "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file2.txt",           "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "collision.txt",       "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision.txt~L1~g~", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__NAME, "value_ancestor")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__NAME, "value_ancestor")
	TEST_VERIFY
		VERIFY_FILE("@/file1.txt", NULL)
		VERIFY_FILE("@/file2.txt", NULL)
		VERIFY_GONE("@/collision.txt")
		VERIFY_TEMP_GONE("@/collision.txt~L0~g~", "item_file1")
		VERIFY_TEMP_GONE("@/collision.txt~L1~g~", "item_file2")
TEST_END

// a test that creates colliding files via renaming
// and accepts the state of all items from "baseline"
TEST("collision__rename__baseline")
	TEST_SETUP
		SETUP_ADD_FILE("@/file1.txt", NULL, NULL, NULL, NULL)
		SETUP_ADD_FILE("@/file2.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file2.txt", "collision.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file1.txt", "collision.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file1.txt", "@/collision.txt", "@/file1.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file1.txt",           "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "collision.txt",       "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file1.txt",           "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision.txt~L0~g~", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file2.txt", "@/file2.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file2.txt",           "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file2.txt",           "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "collision.txt",       "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision.txt~L1~g~", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__NAME, "value_baseline")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__NAME, "value_baseline")
	TEST_VERIFY
		VERIFY_GONE("@/file1.txt")
		VERIFY_FILE("@/file2.txt", NULL)
		VERIFY_FILE("@/collision.txt", NULL)
		VERIFY_TEMP_GONE("@/collision.txt~L0~g~", "item_file1")
		VERIFY_TEMP_GONE("@/collision.txt~L1~g~", "item_file2")
TEST_END

// a test that creates colliding files via renaming
// and accepts the state of all items from "other"
TEST("collision__rename__other")
	TEST_SETUP
		SETUP_ADD_FILE("@/file1.txt", NULL, NULL, NULL, NULL)
		SETUP_ADD_FILE("@/file2.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file2.txt", "collision.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file1.txt", "collision.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file1.txt", "@/collision.txt", "@/file1.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file1.txt",           "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "collision.txt",       "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file1.txt",           "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision.txt~L0~g~", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file2.txt", "@/file2.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file2.txt",           "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file2.txt",           "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "collision.txt",       "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision.txt~L1~g~", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__NAME, "value_other")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__NAME, "value_other")
	TEST_VERIFY
		VERIFY_FILE("@/file1.txt", NULL)
		VERIFY_GONE("@/file2.txt")
		VERIFY_FILE("@/collision.txt", NULL)
		VERIFY_TEMP_GONE("@/collision.txt~L0~g~", "item_file1")
		VERIFY_TEMP_GONE("@/collision.txt~L1~g~", "item_file2")
TEST_END

// a test that creates colliding files via renaming
// and accepts the state of all items from "working"
TEST("collision__rename__working")
	TEST_SETUP
		SETUP_ADD_FILE("@/file1.txt", NULL, NULL, NULL, NULL)
		SETUP_ADD_FILE("@/file2.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file2.txt", "collision.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file1.txt", "collision.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file1.txt", "@/collision.txt", "@/file1.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file1.txt",           "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "collision.txt",       "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file1.txt",           "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision.txt~L0~g~", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file2.txt", "@/file2.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file2.txt",           "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file2.txt",           "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "collision.txt",       "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision.txt~L1~g~", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__NAME, "value_working")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__NAME, "value_working")
	TEST_VERIFY
		VERIFY_GONE("@/file1.txt")
		VERIFY_GONE("@/file2.txt")
		VERIFY_GONE("@/collision.txt")
		VERIFY_TEMP_FILE("@/collision.txt~L0~g~", "item_file1", NULL)
		VERIFY_TEMP_FILE("@/collision.txt~L1~g~", "item_file2", NULL)
TEST_END

// a test that creates colliding files via renaming
// and modifies the WD after merging
// and accepts the state of all items from "ancestor"
TEST("collision__rename_modify__ancestor")
	TEST_SETUP
		SETUP_ADD_FILE("@/file1.txt", "item_file1", NULL, NULL, NULL)
		SETUP_ADD_FILE("@/file2.txt", "item_file2", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file2.txt", "collision.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file1.txt", "collision.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_RENAME("item_file1", "collision1.txt")
		SETUP_RENAME("item_file2", "collision2.txt")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file1.txt", "@/collision.txt", "@/file1.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file1.txt",      "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "collision.txt",  "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file1.txt",      "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision1.txt", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file2.txt", "@/file2.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file2.txt",      "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file2.txt",      "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "collision.txt",  "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision2.txt", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__NAME, "value_ancestor")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__NAME, "value_ancestor")
	TEST_VERIFY
		VERIFY_FILE("@/file1.txt", NULL)
		VERIFY_FILE("@/file2.txt", NULL)
		VERIFY_GONE("@/collision.txt")
		VERIFY_GONE("@/collision1.txt")
		VERIFY_GONE("@/collision2.txt")
		VERIFY_TEMP_GONE("@/collision.txt~L0~g~", "item_file1")
		VERIFY_TEMP_GONE("@/collision.txt~L1~g~", "item_file2")
TEST_END

// a test that creates colliding files via renaming
// and modifies the WD after merging
// and accepts the state of all items from "baseline"
TEST("collision__rename_modify__baseline")
	TEST_SETUP
		SETUP_ADD_FILE("@/file1.txt", "item_file1", NULL, NULL, NULL)
		SETUP_ADD_FILE("@/file2.txt", "item_file2", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file2.txt", "collision.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file1.txt", "collision.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_RENAME("item_file1", "collision1.txt")
		SETUP_RENAME("item_file2", "collision2.txt")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file1.txt", "@/collision.txt", "@/file1.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file1.txt",      "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "collision.txt",  "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file1.txt",      "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision1.txt", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file2.txt", "@/file2.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file2.txt",      "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file2.txt",      "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "collision.txt",  "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision2.txt", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__NAME, "value_baseline")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__NAME, "value_baseline")
	TEST_VERIFY
		VERIFY_GONE("@/file1.txt")
		VERIFY_FILE("@/file2.txt", NULL)
		VERIFY_FILE("@/collision.txt", NULL)
		VERIFY_GONE("@/collision1.txt")
		VERIFY_GONE("@/collision2.txt")
		VERIFY_TEMP_GONE("@/collision.txt~L0~g~", "item_file1")
		VERIFY_TEMP_GONE("@/collision.txt~L1~g~", "item_file2")
TEST_END

// a test that creates colliding files via renaming
// and modifies the WD after merging
// and accepts the state of all items from "other"
TEST("collision__rename_modify__other")
	TEST_SETUP
		SETUP_ADD_FILE("@/file1.txt", "item_file1", NULL, NULL, NULL)
		SETUP_ADD_FILE("@/file2.txt", "item_file2", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file2.txt", "collision.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file1.txt", "collision.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_RENAME("item_file1", "collision1.txt")
		SETUP_RENAME("item_file2", "collision2.txt")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file1.txt", "@/collision.txt", "@/file1.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file1.txt",      "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "collision.txt",  "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file1.txt",      "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision1.txt", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file2.txt", "@/file2.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file2.txt",      "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file2.txt",      "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "collision.txt",  "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision2.txt", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__NAME, "value_other")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__NAME, "value_other")
	TEST_VERIFY
		VERIFY_FILE("@/file1.txt", NULL)
		VERIFY_GONE("@/file2.txt")
		VERIFY_FILE("@/collision.txt", NULL)
		VERIFY_GONE("@/collision1.txt")
		VERIFY_GONE("@/collision2.txt")
		VERIFY_TEMP_GONE("@/collision.txt~L0~g~", "item_file1")
		VERIFY_TEMP_GONE("@/collision.txt~L1~g~", "item_file2")
TEST_END

// a test that creates colliding files via renaming
// and modifies the WD after merging
// and accepts the state of all items from "working"
TEST("collision__rename_modify__working")
	TEST_SETUP
		SETUP_ADD_FILE("@/file1.txt", "item_file1", NULL, NULL, NULL)
		SETUP_ADD_FILE("@/file2.txt", "item_file2", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file2.txt", "collision.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file1.txt", "collision.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_RENAME("item_file1", "collision1.txt")
		SETUP_RENAME("item_file2", "collision2.txt")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file1.txt", "@/collision.txt", "@/file1.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file1.txt",      "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "collision.txt",  "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file1.txt",      "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision1.txt", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file2.txt", "@/file2.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file2.txt",      "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file2.txt",      "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "collision.txt",  "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "collision2.txt", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__NAME, "value_working")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__NAME, "value_working")
	TEST_VERIFY
		VERIFY_GONE("@/file1.txt")
		VERIFY_GONE("@/file2.txt")
		VERIFY_GONE("@/collision.txt")
		VERIFY_FILE("@/collision1.txt", NULL)
		VERIFY_FILE("@/collision2.txt", NULL)
		VERIFY_TEMP_GONE("@/collision.txt~L0~g~", "item_file1")
		VERIFY_TEMP_GONE("@/collision.txt~L1~g~", "item_file2")
TEST_END

// a test that creates colliding files via moving
// and accepts the state of all items from "ancestor"
TEST("collision__move__ancestor")
	TEST_SETUP
		SETUP_ADD_FOLDER("@/folder1", NULL)
		SETUP_ADD_FOLDER("@/folder2", NULL)
		SETUP_ADD_FILE("@/folder1/collision.txt", NULL, NULL, NULL, NULL)
		SETUP_ADD_FILE("@/folder2/collision.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_MOVE("@/folder2/collision.txt", "@/")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_MOVE("@/folder1/collision.txt", "@/")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_folder1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder1/collision.txt", "@/collision.txt", "@/folder1/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder1", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/",        "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/folder1", "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/",        NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_folder2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_folder2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder2/collision.txt", "@/folder2/collision.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder2", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/folder2", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/",        "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/",        NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_folder1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_folder1", SG_RESOLVE__STATE__LOCATION, "value_ancestor")
		RESOLVE_ACCEPT("item_folder2", SG_RESOLVE__STATE__LOCATION, "value_ancestor")
	TEST_VERIFY
		VERIFY_FILE("@/folder1/collision.txt", NULL)
		VERIFY_FILE("@/folder2/collision.txt", NULL)
		VERIFY_GONE("@/collision.txt")
TEST_END

// a test that creates colliding files via moving
// and accepts the state of all items from "baseline"
TEST("collision__move__baseline")
	TEST_SETUP
		SETUP_ADD_FOLDER("@/folder1", NULL)
		SETUP_ADD_FOLDER("@/folder2", NULL)
		SETUP_ADD_FILE("@/folder1/collision.txt", NULL, NULL, NULL, NULL)
		SETUP_ADD_FILE("@/folder2/collision.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_MOVE("@/folder2/collision.txt", "@/")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_MOVE("@/folder1/collision.txt", "@/")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_folder1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder1/collision.txt", "@/collision.txt", "@/folder1/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder1", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/",        "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/folder1", "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/",        NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_folder2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_folder2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder2/collision.txt", "@/folder2/collision.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder2", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/folder2", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/",        "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/",        NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_folder1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_folder1", SG_RESOLVE__STATE__LOCATION, "value_baseline")
		RESOLVE_ACCEPT("item_folder2", SG_RESOLVE__STATE__LOCATION, "value_baseline")
	TEST_VERIFY
		VERIFY_GONE("@/folder1/collision.txt")
		VERIFY_FILE("@/folder2/collision.txt", NULL)
		VERIFY_FILE("@/collision.txt", NULL)
TEST_END

// a test that creates colliding files via moving
// and accepts the state of all items from "other"
TEST("collision__move__other")
	TEST_SETUP
		SETUP_ADD_FOLDER("@/folder1", NULL)
		SETUP_ADD_FOLDER("@/folder2", NULL)
		SETUP_ADD_FILE("@/folder1/collision.txt", NULL, NULL, NULL, NULL)
		SETUP_ADD_FILE("@/folder2/collision.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_MOVE("@/folder2/collision.txt", "@/")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_MOVE("@/folder1/collision.txt", "@/")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_folder1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder1/collision.txt", "@/collision.txt", "@/folder1/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder1", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/",        "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/folder1", "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/",        NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_folder2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_folder2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder2/collision.txt", "@/folder2/collision.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder2", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/folder2", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/",        "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/",        NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_folder1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_folder1", SG_RESOLVE__STATE__LOCATION, "value_other")
		RESOLVE_ACCEPT("item_folder2", SG_RESOLVE__STATE__LOCATION, "value_other")
	TEST_VERIFY
		VERIFY_FILE("@/folder1/collision.txt", NULL)
		VERIFY_GONE("@/folder2/collision.txt")
		VERIFY_FILE("@/collision.txt", NULL)
TEST_END

// a test that creates colliding files via moving
// and modifies the WD after merging
// and accepts the state of all items from "ancestor"
TEST("collision__move_modify__ancestor")
	TEST_SETUP
		SETUP_ADD_FOLDER("@/folder1", NULL)
		SETUP_ADD_FOLDER("@/folder2", NULL)
		SETUP_ADD_FILE("@/folder1/collision.txt", "item_file1", NULL, NULL, NULL)
		SETUP_ADD_FILE("@/folder2/collision.txt", "item_file2", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_MOVE("@/folder2/collision.txt", "@/")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_MOVE("@/folder1/collision.txt", "@/")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_ADD_FOLDER("@/folder3", NULL)
		SETUP_MOVE("item_file1", "@/folder3")
		SETUP_ADD_FOLDER("@/folder4", NULL)
		SETUP_MOVE("item_file2", "@/folder4")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder1/collision.txt", "@/collision.txt", "@/folder1/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder1", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/",        "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/folder1", "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/folder3", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder2/collision.txt", "@/folder2/collision.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder2", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/folder2", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/",        "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/folder4", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__LOCATION, "value_ancestor")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__LOCATION, "value_ancestor")
	TEST_VERIFY
		VERIFY_FILE("@/folder1/collision.txt", NULL)
		VERIFY_FILE("@/folder2/collision.txt", NULL)
		VERIFY_GONE("@/collision.txt")
		VERIFY_GONE("@/folder3/collision.txt")
		VERIFY_GONE("@/folder4/collision.txt")
TEST_END

// a test that creates colliding files via moving
// and modifies the WD after merging
// and accepts the state of all items from "baseline"
TEST("collision__move_modify__baseline")
	TEST_SETUP
		SETUP_ADD_FOLDER("@/folder1", NULL)
		SETUP_ADD_FOLDER("@/folder2", NULL)
		SETUP_ADD_FILE("@/folder1/collision.txt", "item_file1", NULL, NULL, NULL)
		SETUP_ADD_FILE("@/folder2/collision.txt", "item_file2", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_MOVE("@/folder2/collision.txt", "@/")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_MOVE("@/folder1/collision.txt", "@/")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_ADD_FOLDER("@/folder3", NULL)
		SETUP_MOVE("item_file1", "@/folder3")
		SETUP_ADD_FOLDER("@/folder4", NULL)
		SETUP_MOVE("item_file2", "@/folder4")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder1/collision.txt", "@/collision.txt", "@/folder1/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder1", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/",        "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/folder1", "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/folder3", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder2/collision.txt", "@/folder2/collision.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder2", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/folder2", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/",        "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/folder4", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__LOCATION, "value_baseline")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__LOCATION, "value_baseline")
	TEST_VERIFY
		VERIFY_GONE("@/folder1/collision.txt")
		VERIFY_FILE("@/folder2/collision.txt", NULL)
		VERIFY_FILE("@/collision.txt", NULL)
		VERIFY_GONE("@/folder3/collision.txt")
		VERIFY_GONE("@/folder4/collision.txt")
TEST_END

// a test that creates colliding files via moving
// and modifies the WD after merging
// and accepts the state of all items from "other"
TEST("collision__move_modify__other")
	TEST_SETUP
		SETUP_ADD_FOLDER("@/folder1", NULL)
		SETUP_ADD_FOLDER("@/folder2", NULL)
		SETUP_ADD_FILE("@/folder1/collision.txt", "item_file1", NULL, NULL, NULL)
		SETUP_ADD_FILE("@/folder2/collision.txt", "item_file2", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_MOVE("@/folder2/collision.txt", "@/")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_MOVE("@/folder1/collision.txt", "@/")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_ADD_FOLDER("@/folder3", NULL)
		SETUP_MOVE("item_file1", "@/folder3")
		SETUP_ADD_FOLDER("@/folder4", NULL)
		SETUP_MOVE("item_file2", "@/folder4")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder1/collision.txt", "@/collision.txt", "@/folder1/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder1", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/",        "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/folder1", "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/folder3", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder2/collision.txt", "@/folder2/collision.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder2", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/folder2", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/",        "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/folder4", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__LOCATION, "value_other")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__LOCATION, "value_other")
	TEST_VERIFY
		VERIFY_FILE("@/folder1/collision.txt", NULL)
		VERIFY_GONE("@/folder2/collision.txt")
		VERIFY_FILE("@/collision.txt", NULL)
		VERIFY_GONE("@/folder3/collision.txt")
		VERIFY_GONE("@/folder4/collision.txt")
TEST_END

// a test that creates colliding files via moving
// and modifies the WD after merging
// and accepts the state of all items from "working"
TEST("collision__move_modify__working")
	TEST_SETUP
		SETUP_ADD_FOLDER("@/folder1", NULL)
		SETUP_ADD_FOLDER("@/folder2", NULL)
		SETUP_ADD_FILE("@/folder1/collision.txt", "item_file1", NULL, NULL, NULL)
		SETUP_ADD_FILE("@/folder2/collision.txt", "item_file2", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_MOVE("@/folder2/collision.txt", "@/")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_MOVE("@/folder1/collision.txt", "@/")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_ADD_FOLDER("@/folder3", NULL)
		SETUP_MOVE("item_file1", "@/folder3")
		SETUP_ADD_FOLDER("@/folder4", NULL)
		SETUP_MOVE("item_file2", "@/folder4")
	TEST_EXPECT
		EXPECT_ITEM("item_file1", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder1/collision.txt", "@/collision.txt", "@/folder1/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder1", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/",        "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/folder1", "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/folder3", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file2")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_file2", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/folder2/collision.txt", "@/folder2/collision.txt", "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__LOCATION, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "@/folder2", "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "@/folder2", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "@/",        "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "@/folder4", NULL)
				EXPECT_RELATED_BEGIN
					EXPECT_RELATED_COLLISION("item_file1")
				EXPECT_RELATED_END
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file1", SG_RESOLVE__STATE__LOCATION, "value_working")
		RESOLVE_ACCEPT("item_file2", SG_RESOLVE__STATE__LOCATION, "value_working")
	TEST_VERIFY
		VERIFY_GONE("@/folder1/collision.txt")
		VERIFY_GONE("@/folder2/collision.txt")
		VERIFY_GONE("@/collision.txt")
		VERIFY_FILE("@/folder3/collision.txt", NULL)
		VERIFY_FILE("@/folder4/collision.txt", NULL)
TEST_END

// a test that creates colliding files via addition
// and accepts the state of all items from "baseline"
TEST("collision__add__baseline")
	TEST_SETUP
		SETUP_ADD_FILE("@/dummy.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_ADD_FILE("@/collision.txt", NULL, "contents_other", _content__append_numbered_lines, "7")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_ADD_FILE("@/collision.txt", NULL, "contents_baseline", _content__append_numbered_lines, "3")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_baseline", SG_TREENODEENTRY_TYPE_REGULAR_FILE, NULL, "@/collision.txt", NULL, SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__EXISTENCE, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__BOOL("value_baseline", SG_TRUE,  "commit_baseline")
				EXPECT_CHANGESET_VALUE__BOOL("value_other",    SG_FALSE, "commit_other")
				EXPECT_CHANGESET_VALUE__BOOL("value_working",  SG_TRUE,  NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_other", SG_TREENODEENTRY_TYPE_REGULAR_FILE, NULL, NULL, "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__EXISTENCE, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__BOOL("value_baseline", SG_FALSE, "commit_baseline")
				EXPECT_CHANGESET_VALUE__BOOL("value_other",    SG_TRUE,  "commit_other")
				EXPECT_CHANGESET_VALUE__BOOL("value_working",  SG_TRUE,  NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_baseline", SG_RESOLVE__STATE__EXISTENCE, "value_baseline")
		RESOLVE_ACCEPT("item_other",    SG_RESOLVE__STATE__EXISTENCE, "value_baseline")
	TEST_VERIFY
		VERIFY_FILE("@/dummy.txt", NULL)
		VERIFY_FILE("@/collision.txt", "contents_baseline")
TEST_END

// a test that creates colliding files via addition
// and accepts the state of all items from "other"
TEST("collision__add__other")
	TEST_SETUP
		SETUP_ADD_FILE("@/dummy.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_ADD_FILE("@/collision.txt", NULL, "contents_other", _content__append_numbered_lines, "7")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_ADD_FILE("@/collision.txt", NULL, "contents_baseline", _content__append_numbered_lines, "3")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_baseline", SG_TREENODEENTRY_TYPE_REGULAR_FILE, NULL, "@/collision.txt", NULL, SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__EXISTENCE, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__BOOL("value_baseline", SG_TRUE,  "commit_baseline")
				EXPECT_CHANGESET_VALUE__BOOL("value_other",    SG_FALSE, "commit_other")
				EXPECT_CHANGESET_VALUE__BOOL("value_working",  SG_TRUE,  NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
		EXPECT_ITEM("item_other", SG_TREENODEENTRY_TYPE_REGULAR_FILE, NULL, NULL, "@/collision.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__EXISTENCE, 0, SG_TRUE, NULL)
				EXPECT_CHANGESET_VALUE__BOOL("value_baseline", SG_FALSE, "commit_baseline")
				EXPECT_CHANGESET_VALUE__BOOL("value_other",    SG_TRUE,  "commit_other")
				EXPECT_CHANGESET_VALUE__BOOL("value_working",  SG_TRUE,  NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_baseline", SG_RESOLVE__STATE__EXISTENCE, "value_other")
		RESOLVE_ACCEPT("item_other",    SG_RESOLVE__STATE__EXISTENCE, "value_other")
	TEST_VERIFY
		VERIFY_FILE("@/dummy.txt", NULL)
		VERIFY_FILE("@/collision.txt", "contents_other")
TEST_END
