/*
Copyright 2010-2011 SourceGear, LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Resolve Tests
 *
 * This file contains tests that setup divergent rename conflicts.
 */

// a simple divergent rename that accepts the "ancestor" value
TEST("divergent_rename__simple__ancestor")
	TEST_SETUP
		SETUP_ADD_FILE("@/file.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file.txt", "file_other.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file.txt", "file_baseline.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_file", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file.txt", "@/file_baseline.txt", "@/file_other.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME, SG_FALSE, ACCEPTED_VALUE_NONE)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file.txt",                "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file_baseline.txt",       "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file_other.txt",          "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "file_baseline.txt~L0~g~", NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file", SG_RESOLVE__STATE__NAME, "value_ancestor")
	TEST_VERIFY
		VERIFY_FILE("@/file.txt", NULL)
		VERIFY_GONE("@/file_baseline.txt")
		VERIFY_GONE("@/file_other.txt")
		VERIFY_TEMP_GONE("@/file_baseline.txt~L0~g~", "item_file")
TEST_END

// a simple divergent rename that accepts the "baseline" value
TEST("divergent_rename__simple__baseline")
	TEST_SETUP
		SETUP_ADD_FILE("@/file.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file.txt", "file_other.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file.txt", "file_baseline.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_file", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file.txt", "@/file_baseline.txt", "@/file_other.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME, SG_FALSE, ACCEPTED_VALUE_NONE)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file.txt",                "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file_baseline.txt",       "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file_other.txt",          "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "file_baseline.txt~L0~g~", NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file", SG_RESOLVE__STATE__NAME, "value_baseline")
	TEST_VERIFY
		VERIFY_GONE("@/file.txt")
		VERIFY_FILE("@/file_baseline.txt", NULL)
		VERIFY_GONE("@/file_other.txt")
		VERIFY_TEMP_GONE("@/file_baseline.txt~L0~g~", "item_file")
TEST_END

// a simple divergent rename that accepts the "other" value
TEST("divergent_rename__simple__other")
	TEST_SETUP
		SETUP_ADD_FILE("@/file.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file.txt", "file_other.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file.txt", "file_baseline.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_file", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file.txt", "@/file_baseline.txt", "@/file_other.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME, SG_FALSE, ACCEPTED_VALUE_NONE)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file.txt",                "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file_baseline.txt",       "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file_other.txt",          "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "file_baseline.txt~L0~g~", NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file", SG_RESOLVE__STATE__NAME, "value_other")
	TEST_VERIFY
		VERIFY_GONE("@/file.txt")
		VERIFY_GONE("@/file_baseline.txt")
		VERIFY_FILE("@/file_other.txt", NULL)
		VERIFY_TEMP_GONE("@/file_baseline.txt~L0~g~", "item_file")
TEST_END

// a simple divergent rename that accepts the "working" value
TEST("divergent_rename__simple__working")
	TEST_SETUP
		SETUP_ADD_FILE("@/file.txt", NULL, NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file.txt", "file_other.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file.txt", "file_baseline.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
	TEST_EXPECT
		EXPECT_ITEM("item_file", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file.txt", "@/file_baseline.txt", "@/file_other.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME, SG_FALSE, ACCEPTED_VALUE_NONE)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file.txt",                "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file_baseline.txt",       "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file_other.txt",          "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "file_baseline.txt~L0~g~", NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file", SG_RESOLVE__STATE__NAME, "value_working")
	TEST_VERIFY
		VERIFY_GONE("@/file.txt")
		VERIFY_GONE("@/file_baseline.txt")
		VERIFY_GONE("@/file_other.txt")
		VERIFY_TEMP_FILE("@/file_baseline.txt~L0~g~", "item_file", NULL)
TEST_END

// a divergent rename that modifies the WD before accepting the "ancestor" value
TEST("divergent_rename__modify__ancestor")
	TEST_SETUP
		SETUP_ADD_FILE("@/file.txt", "item_file", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file.txt", "file_other.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file.txt", "file_baseline.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_RENAME("item_file", "file_working.txt")
	TEST_EXPECT
		EXPECT_ITEM("item_file", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file.txt", "@/file_baseline.txt", "@/file_other.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME, SG_FALSE, ACCEPTED_VALUE_NONE)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file.txt",          "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file_baseline.txt", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file_other.txt",    "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "file_working.txt",  NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file", SG_RESOLVE__STATE__NAME, "value_ancestor")
	TEST_VERIFY
		VERIFY_FILE("@/file.txt", NULL)
		VERIFY_GONE("@/file_baseline.txt")
		VERIFY_GONE("@/file_other.txt")
		VERIFY_GONE("@/file_working.txt")
		VERIFY_TEMP_GONE("@/file_baseline.txt~L0~g~", "item_file")
TEST_END

// a divergent rename that modifies the WD before accepting the "baseline" value
TEST("divergent_rename__modify__baseline")
	TEST_SETUP
		SETUP_ADD_FILE("@/file.txt", "item_file", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file.txt", "file_other.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file.txt", "file_baseline.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_RENAME("item_file", "file_working.txt")
	TEST_EXPECT
		EXPECT_ITEM("item_file", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file.txt", "@/file_baseline.txt", "@/file_other.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME, SG_FALSE, ACCEPTED_VALUE_NONE)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file.txt",          "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file_baseline.txt", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file_other.txt",    "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "file_working.txt",  NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file", SG_RESOLVE__STATE__NAME, "value_baseline")
	TEST_VERIFY
		VERIFY_GONE("@/file.txt")
		VERIFY_FILE("@/file_baseline.txt", NULL)
		VERIFY_GONE("@/file_other.txt")
		VERIFY_GONE("@/file_working.txt")
		VERIFY_TEMP_GONE("@/file_baseline.txt~L0~g~", "item_file")
TEST_END

// a divergent rename that modifies the WD before accepting the "other" value
TEST("divergent_rename__modify__other")
	TEST_SETUP
		SETUP_ADD_FILE("@/file.txt", "item_file", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file.txt", "file_other.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file.txt", "file_baseline.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_RENAME("item_file", "file_working.txt")
	TEST_EXPECT
		EXPECT_ITEM("item_file", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file.txt", "@/file_baseline.txt", "@/file_other.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME, SG_FALSE, ACCEPTED_VALUE_NONE)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file.txt",          "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file_baseline.txt", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file_other.txt",    "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "file_working.txt",  NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file", SG_RESOLVE__STATE__NAME, "value_other")
	TEST_VERIFY
		VERIFY_GONE("@/file.txt")
		VERIFY_GONE("@/file_baseline.txt")
		VERIFY_FILE("@/file_other.txt", NULL)
		VERIFY_GONE("@/file_working.txt")
		VERIFY_TEMP_GONE("@/file_baseline.txt~L0~g~", "item_file")
TEST_END

// a divergent rename that modifies the WD before accepting the "working" value
TEST("divergent_rename__modify__working")
	TEST_SETUP
		SETUP_ADD_FILE("@/file.txt", "item_file", NULL, NULL, NULL)
		SETUP_COMMIT("commit_ancestor")

		SETUP_RENAME("@/file.txt", "file_other.txt")
		SETUP_COMMIT("commit_other")

		SETUP_UPDATE("commit_ancestor")
		SETUP_RENAME("@/file.txt", "file_baseline.txt")
		SETUP_COMMIT("commit_baseline")

		SETUP_MERGE("commit_other")
		SETUP_RENAME("item_file", "file_working.txt")
	TEST_EXPECT
		EXPECT_ITEM("item_file", SG_TREENODEENTRY_TYPE_REGULAR_FILE, "@/file.txt", "@/file_baseline.txt", "@/file_other.txt", SG_FALSE)
			EXPECT_CHOICE(SG_RESOLVE__STATE__NAME, SG_MRG_CSET_ENTRY_CONFLICT_FLAGS__DIVERGENT_RENAME, SG_FALSE, ACCEPTED_VALUE_NONE)
				EXPECT_CHANGESET_VALUE__SZ("value_ancestor", "file.txt",          "commit_ancestor")
				EXPECT_CHANGESET_VALUE__SZ("value_baseline", "file_baseline.txt", "commit_baseline")
				EXPECT_CHANGESET_VALUE__SZ("value_other",    "file_other.txt",    "commit_other")
				EXPECT_CHANGESET_VALUE__SZ("value_working",  "file_working.txt",  NULL)
				EXPECT_RELATED_NONE
			EXPECT_CHOICE_END
		EXPECT_ITEM_END
	TEST_RESOLVE
		RESOLVE_ACCEPT("item_file", SG_RESOLVE__STATE__NAME, "value_working")
	TEST_VERIFY
		VERIFY_GONE("@/file.txt")
		VERIFY_GONE("@/file_baseline.txt")
		VERIFY_GONE("@/file_other.txt")
		VERIFY_FILE("@/file_working.txt", NULL)
		VERIFY_TEMP_GONE("@/file_baseline.txt~L0~g~", "item_file")
TEST_END

